import boto3
import os

def download_copernicus_dem(region_coords, resolution='30m', output_dir='./madinah_dem'):
    """
    Download Copernicus DEM for a specified region using AWS Open Data.

    Parameters:
        region_coords (list): List of bounding box coordinates [min_lon, min_lat, max_lon, max_lat].
        resolution (str): Resolution of the DEM ('30m' or '90m').
        output_dir (str): Directory to save the downloaded DEM files.
    """
    # AWS S3 bucket for Copernicus DEM
    bucket_name = 'copernicus-dem-30m' if resolution == '30m' else 'copernicus-dem-90m'
    prefix = 'Copernicus_DSM_COG_10_'  # Path structure in the bucket

    # Ensure the output directory exists
    os.makedirs(output_dir, exist_ok=True)

    # Initialize S3 client
    s3 = boto3.client('s3')

    # List objects in the bucket within the specified resolution folder
    paginator = s3.get_paginator('list_objects_v2')
    pages = paginator.paginate(Bucket=bucket_name, Prefix=prefix)

    print("Fetching DEM files from AWS S3 bucket...")
    for page in pages:
        if 'Contents' not in page:
            continue

        for obj in page['Contents']:
            key = obj['Key']

            # Check if the tile intersects with the region (basic filtering using naming conventions)
            lon_lat_info = key.split('_')[-2:]  # Extract longitude and latitude info
            tile_lon = int(lon_lat_info[0])
            tile_lat = int(lon_lat_info[1].split('.')[0])

            # Filter tiles that fall within the bounding box
            min_lon, min_lat, max_lon, max_lat = region_coords
            if min_lon <= tile_lon <= max_lon and min_lat <= tile_lat <= max_lat:
                # Download the file
                file_name = os.path.join(output_dir, os.path.basename(key))
                print(f"Downloading {file_name}...")
                s3.download_file(bucket_name, key, file_name)

    print(f"DEM files downloaded to {output_dir}")

if __name__ == "__main__":
    # Define bounding box for Madinah region (example: approximate values in EPSG:4326)
    madinah_bbox = [37.074437910243304, 22.512577028172675, 42.1950402673047, 27.496387330433468]  # [min_lon, min_lat, max_lon, max_lat]

    # Call the function to download the DEM
    download_copernicus_dem(madinah_bbox, resolution='30m')
